// ***************************************
// Plugin Processor Clock timer for SourceBoost IDE
// 
// Author(s): David Hobday
//
// Copyright (C) 2003-2010 Pavel Baranov
// Copyright (C) 2003-2010 David Hobday
// 
// This code may be freely used to develop
// Plugins for the SourceBoost IDE
// ***************************************

#ifndef _CLK_TIMER_
#define _CLK_TIMER_

#include "pluginAPI.h"

class CClkTimer
{
public:
	CClkTimer()
	{
		m_finishTime_clk = 0;
		m_clkRate_Hz = 0;
		m_done = false;
		m_running = false;
	}

	///
	/// Set clock rate in Hz.
	/// Used when converting from standard units to clock count
	///
	void SetClkRate( long rate_Hz )
	{
		m_clkRate_Hz = rate_Hz;
	}

	///
	/// Set the timer value in micro seconds
	/// This value will be used on the next call of the start function
	///
	void Presetus( long time_us )
	{
		m_delay_clk = (long)(time_us/1000000.0 * m_clkRate_Hz);
	}
	///
	/// Set the timer value in milli seconds
	/// This value will be used on the next call of the start function
	///
	void Presetms( long time_ms )
	{
		m_delay_clk = (long)(time_ms/1000.0 * m_clkRate_Hz);
	}
	///
	/// Set the timer value in clock ticks
	/// This value will be used on the next call of the start function
	///
	void PresetClk( long time_clk )
	{
		m_delay_clk = time_clk;
	}

	///
	/// Start a timing operation
	/// This value will be used on the next call of the start function
	///
	void Start() 
	{ 
		long now; 
		GetClk( &now ); 
		m_finishTime_clk = now + m_delay_clk;
		m_done = false;
		m_running = true;
	}

	///
	/// Stop a timing operation before it completes.
	/// This stops the timer ever being done 
	///
	void Stop()
	{
		m_running = false;
	}

	///
	/// Done tests the timer to see if the timing operation is complete.
	///
	bool Done()
	{
		if ( m_done )
			return true;
		
		if ( ! m_running )
			return false;

		long now; 
		GetClk( &now ); 
		if ( now - m_finishTime_clk >= 0 )
		{
			m_done = true;
			m_running = false;
		}

		return m_done;
	}

	///
	/// Done tests the timer to see if the timing operation is complete.
	/// Uses now argument to save calling GetClk
	///
	bool Done( long now )
	{
		if ( m_done )
			return true;
		
		if ( ! m_running )
			return false;

		if ( now - m_finishTime_clk >= 0 )
		{
			m_done = true;
			m_running = false;
		}

		return m_done;
	}

	///
	/// Function to save internal state of timer, useful for backstep implementation
	///
	ostream& Save( ostream& os )
	{
		os.write( (const char*)&m_finishTime_clk, sizeof( m_finishTime_clk ) );
		os.write( (const char*)&m_delay_clk, sizeof( m_delay_clk ) );
		os.write( (const char*)&m_clkRate_Hz, sizeof( m_clkRate_Hz ) );
		os.write( (const char*)&m_done, sizeof( m_done ) );
		os.write( (const char*)&m_running, sizeof( m_running ) );
		return os;
	}

	///
	/// Function to load internal state of timer, useful for backstep implementation
	///
	istream& Load( istream& is )
	{
		is.read( (char*)&m_finishTime_clk, sizeof( m_finishTime_clk ) );
		is.read( (char*)&m_delay_clk, sizeof( m_delay_clk ) );
		is.read( (char*)&m_clkRate_Hz, sizeof( m_clkRate_Hz ) );
		is.read( (char*)&m_done, sizeof( m_done ) );
		is.read( (char*)&m_running, sizeof( m_running ) );
		return is;
	}

private:
	long m_finishTime_clk;
	long m_delay_clk;
	long m_clkRate_Hz;
	bool m_done;
	bool m_running;
};


#endif // _CLK_TIMER_