// ***************************************
// Plugin Data holding class for SourceBoost IDE
// 
// Author(s): David Hobday
//
// Copyright (C) 2003-2010 Pavel Baranov
// Copyright (C) 2003-2010 David Hobday
// 
// This code may be freely used to develop
// Plugins for the SourceBoost IDE
// ***************************************

#include "stdafx.h"

#include "PluginApp.h"
#include <strstrea.h>
#include <math.h>

extern CPluginApp app;

///////////////////////////////////////////////////////////
// Functions called in sync with sim thread so no locking
// required. No locking mean faster execution!
///////////////////////////////////////////////////////////

CPluginData::CPluginData()
{
	// electrical characteristic
	// used as output with 5 ohm
	// -1 denote infinite resistance when in input mode
	// and when input internal voltage source is 0V
	m_pin.Configure( -1, 0, 5 );
	m_pin.MakeOutput();
}

CPluginData::~CPluginData()
{
}

void CPluginData::Start()
{
	m_pin.Connect( m_pinName );
}

void CPluginData::Stop()
{
	m_pin.Disconnect();
}


////////////////////////////////////////////////////////////
// SaveState/LoadState code below should save any internal 
// data that will need to be restored.
// SaveState gets called when forward stepping is performed.
// LoadState gets called when backward stepping is performed, to restore the state to a previous value.

void CPluginData::SaveState( ostream& os )
{
	// save data that will be needed to restore plugin state to an "as it was state"
	
	// save pin voltage
	os.write( (char*)&m_VoutmV, sizeof (m_VoutmV) );
}

void CPluginData::LoadState( istream& os )
{
	// restore data as it was previously

	// restore pin voltage
	os.read( (char*)&m_VoutmV, sizeof (m_VoutmV) );

	// update pin to outside world
	PinUpdate();

	// plugin view update in case anything has changed
	SendNotifyMessage( *app.view, CPluginView::WMU_DATA_REQ_UPDATE, 0, 0 );
}

void CPluginData::Step( short sDirection )
{
}

void CPluginData::Instr( long clk )
{
}

void CPluginData::Reset()
{
}


void CPluginData::Clk( long clk )
{
}

///////////////////////////////////////////////////////////
// Functions that will be called by GUI thread
// lock and unlock essential
///////////////////////////////////////////////////////////

// this function gets called as a result of an SourceBoost IDE callback
void CPluginData::SetClkRate( long clkRate )
{
	Lock();

	// Store clock rate here if required.
	// Clock rate is useful to allow conversion from a processor clock count to 'real time'

	Unlock();
}

void CPluginData::SetPinName( const char* pinName )
{
	Lock();

	m_pinName = pinName;

	Unlock();
}

void CPluginData::GetPinName( CString& pinName )
{
	Lock();

	pinName = m_pinName;

	Unlock();
}

void CPluginData::Connect()
{		
	Lock();

	// disconnect if currently connected	
	m_pin.Disconnect(); 

	// connect to the required pin
	m_pin.Connect( m_pinName ); 
	
		
	// update the pin to refelect current settings
	PinUpdate(); 

	Unlock();
}

void CPluginData::Disconnect()
{
	Lock();

	m_pin.Disconnect();

	Unlock();
}

void CPluginData::SetV( int Vmv )
{
	Lock();

	m_VoutmV = Vmv;
	PinUpdate();

	Unlock();	
}

int CPluginData::GetV()
{
	Lock();

	int Vmv;
	Vmv = m_VoutmV;

	Unlock();	

	return Vmv;
}

///////////////////////////////////////////////////////////
// Lock and Unlock functions,
// These functions effectively pause sim thread at the same
// point making sim access safe and predictable :)
///////////////////////////////////////////////////////////
void CPluginData::Lock()
{
	::Lock();
}

void CPluginData::Unlock()
{
	::Unlock();
}

///////////////////////////////////////////////////////////
// Internal (private) functions, only called from function that 
// apply locks if necessary
///////////////////////////////////////////////////////////
void CPluginData::PinUpdate()
{
	m_pin.SetOutputVoltage( m_VoutmV );
}
